// server.js
const express = require('express');
const http = require('http');
const cors = require('cors');
const { Server } = require('socket.io');

const app = express();
app.use(cors({ origin: "http://localhost:5173" })); // Autorise le client Vite
const server = http.createServer(app);
const io = new Server(server, {
  cors: {
    origin: "http://localhost:5173",
    methods: ["GET", "POST"]
  }
});

// Objet pour stocker les joueurs par salle
const players = {};

// Optionnel : servir des fichiers statiques
app.use(express.static('public'));

io.on('connection', (socket) => {
  console.log('Nouvelle connexion : ' + socket.id);

  socket.on('joinGame', (data) => {
    const { room, playerInfo } = data;
    socket.join(room);

    // Initialiser la salle si nécessaire
    if (!players[room]) {
      players[room] = {};
    }
    // Enregistrer le joueur
    players[room][socket.id] = playerInfo;
    console.log(`${socket.id} a rejoint la salle ${room}`);

    // Envoyer à la socket nouvellement connectée la liste des joueurs déjà présents
    const existing = [];
    for (const id in players[room]) {
      if (id !== socket.id) {
        existing.push({ id, playerInfo: players[room][id] });
      }
    }
    socket.emit('existingPlayers', existing);

    // Notifier les autres joueurs de l'arrivée du nouveau joueur
    socket.to(room).emit('newPlayer', { id: socket.id, playerInfo });

    // Vérifier le nombre de joueurs dans la salle
    const roomSize = Object.keys(players[room]).length;
    if (roomSize >= 2) {
      io.to(room).emit('readyToStart', { room });
    }
  });

  socket.on('playerUpdate', (data) => {
    const { room, playerData } = data;
    if (players[room]) {
      players[room][socket.id] = playerData;
    }
    socket.to(room).emit('playerUpdate', { id: socket.id, playerData });
  });

  socket.on('shoot', (data) => {
    const { room, shootData } = data;
    socket.to(room).emit('shoot', { id: socket.id, shootData });
  });

  socket.on('disconnect', () => {
    console.log('Déconnexion : ' + socket.id);
    socket.rooms.forEach((room) => {
      if (room !== socket.id && players[room]) {
        delete players[room][socket.id];
        socket.to(room).emit('playerDisconnected', { id: socket.id });
      }
    });
  });
});

const PORT = process.env.PORT || 3000;
server.listen(PORT, () => console.log(`Serveur démarré sur le port ${PORT}`));
